pro COMMAH, COM=COM, MAH=MAH

;+
; NAME:
;       commah   'COncentration-Mass relation and Mass Accretion History'
;
; PURPOSE:
;        This IDL program computes the mass accretion history (MAH) for 
;        a dark matter halo of mass M_0 and the NFW concentration mass 
;        relation (COM) at redshift z_0. In addition, it also computes 
;        the dark matter accretion rate as well as the integral of the 
;        NFW density profile, suitable for DM annihilation calculations.
;
; CALLING SEQUENCE:
;       resolve_routine, "cosmo_routines", /compile_full_file, /either
;       resolve_routine, "subroutines", /compile_full_file, /either
;       commah
; 
; KEYWORD PARAMETERS:
;       COM : outputs concentration-mass relation
;       MAH : outputs mass accretion history
;
; OUTPUTS:
;       The output is organized as follows:
;       -It is called `MAH_***.dat' if the MAH option is selected. 
;       It outputs: 
;        column 1: Redshift (z) 
;        column 2: Halo mass (M200) at z in units of Log10[Msun] 
;        column 3: Accretion rate at z in units of Log10[Msun/yr] 
;        column 4: Concentration (c200) at z
;        column 5: Virial radius r200 in units of Log10[Mpc]
;        column 6: NFW scale radius r_-2 in units of Log10[Mpc]
;
;       -The output file is called `COM_***.dat' if the COM option is 
;       selected. It outputs:
;        column 1: Concentration (c200)
;        column 2: Halo mass (M200) in units of Log10[Msun]
;        column 3: Rms of the primordial density perturbation field
;        column 4: Peak height
;        column 5: Accretion rate in units of Log10[Msun/yr]
;
;       MAH_WMAP5_M0_12.00_z0_0.0.dat : example output file
;       (corresponds to MAH.in input file)
;       COM_WMAP5_z_0.0.dat           : example output file
;       (corresponds to COM.in input file)
;
; COMMON BLOCKS:
;      Cosmology: cosmovalues
;
; RESTRICTIONS:
;      
;
; PROCEDURE:
;      Type run with your desire output option : COM or MAH
;      $IDL>  run,/COM
;      $IDL>  run,/MAH
;
; MODIFICATION HISTORY:
;      Written by: Camila Correa (correac@student.unimelb.edu.au), 2014
;

common cosmovalues,Om,Ol,h,n,sigma8,Aparam

;; We now decide what to calculate depending on the selected option
if KEYWORD_SET(MAH) then begin
    starttime = systime(1)

;; Read the parameter file
    readcol,'./MAH.in',cosmo,M0,z0,zmax,step,format='(a,d,d,d,d)'
    print,'*********************************************************'
    print,'  This IDL program computes the mass accretion history'
    print,'  for a dark matter halo of mass M 10^{'+string(format='(f0.2)',alog10(M0))+'} Msun,'
    print,'  at redshift z '+string(format='(f0.1)',z0)+', assuming the '+cosmo+' cosmology.'
    print,'*********************************************************'

    M0 = alog10(string(format='(f0.2)',M0))
    z0 = double(string(format='(f0.2)',z0)) 
    zmax = double(string(format='(f0.2)',zmax)) 
    step = double(string(format='(f0.2)',step))

;; Open output file
   file = './MAH_'+cosmo+'_M0_'+string(format='(f0.2)',M0)+'_z0_'+string(format='(f0.1)',z0)+'.dat'
   openw, unitout, file, /get_lun, /more

;; Calculate cosmology related values
   cosmology,cosmo

;; Create a look up table for power spectrum and rms density field
   starttime_table = systime(1)
   print,'Creating sigma look-up table'
   fill_sigma_table
   endtime_table = systime(1)
   print, "sigma table took ", endtime_table-starttime_table, " seconds"

;; Initialize redshift array: 
   num = (zmax-z0)/step+1
   z = findgen(num)*step+z0


   printf,unitout,'# Cosmology (flat), Om : '+string(format='(f0.3)',Om)+',Ol : '+string(format='(f0.3)',Ol)+', h : '+string(format='(f0.2)',h)+', sigma8 : '+string(format='(f0.3)',sigma8)+', ns : '+string(format='(f0.2)',n)+''
   printf,unitout,'#   z   -    Halo     -    Accretion   - Concentration - Virial radius - Scale radius '
   printf,unitout,'#       -    mass     -      rate      -               -               -              '
   printf,unitout,'#       -   (M200)    -  (dM200/dt)    -    (c200)     -     (r200)    -              '
   printf,unitout,'#       - Log10[Msun] - Log10[Msun/yr] -               -  Log10[Mpc]   -  Log10[Mpc]  '

    for j = 0,n_elements(z)-1 do begin
       zj = z[j]
       Mj = MAH(M0,zj,z0)                         ; Halo mass at zj in unit of Msun
       params = COM(Mj,zj)
       cj = params[0]                             ; Concentration: c200
       dMdtj = accretion_rate(M0,zj,z0)           ; Accretion rate in units of Log10[Msun/yr]
       intrhoj = int_rho(Mj,cj,zj)+alog10(h^4.)   ; Log10 Int rho^2 in units of [Msun^2/Mpc^6]
       r200j = r200(Mj,zj)-3*alog10(h)            ; Virial radius, r200, in units of Log10[Mpc]
       r2j = r200j-alog10(cj)                     ; NFW scale radius in units of Log10[Mpc]
       printf, unitout, zj, Mj, dMdtj, cj, r200j, r2j, format='(2X,f0.3,5X,f0.3,9X,f0.3,10X,f0.3,10X,f0.3,10X,f0.3)' 
    endfor   
    close,/all    

    endtime = systime(1)
    print, "Determining halo mass accretion history took ", endtime-starttime, " seconds"

endif
;%/ ----------------------------------------------------

if KEYWORD_SET(COM) then begin
    starttime = systime(1)
;; Read the parameter file
    readcol,'./COM.in',cosmo,z0,Mmin,Mmax,step,format='(a,d,d,d,d)'

    print,'*********************************************************'
    print,'  This IDL program computes the NFW concentration'
    print,'  masss relation at redshift z '+string(format='(f0.1)',z0)  
    print,'  assuming the '+cosmo+' cosmology.'
    print,'*********************************************************'

    z0 = double(string(format='(f0.2)',z0)) 
    Mmin = alog10(string(format='(f0.2)',Mmin))
    Mmax = alog10(string(format='(f0.2)',Mmax)) 
    step = double(string(format='(f0.2)',step))

;; Open output file
    file = './COM_'+cosmo+'_z_'+string(format='(f0.1)',z0)+'.dat'
    openw, unitout, file, /get_lun, /more

;; Calculate cosmology related values
    cosmology,cosmo

   printf,unitout,'# Cosmology (flat), Om : '+string(format='(f0.3)',Om)+',Ol : '+string(format='(f0.3)',Ol)+', h : '+string(format='(f0.2)',h)+', sigma8 : '+string(format='(f0.3)',sigma8)+', ns : '+string(format='(f0.2)',n)+''
   printf,unitout,'# Concentration -    Halo     -    Mass    -   Peak    -   Accretion    '
   printf,unitout,'#               -    mass     -  Variance  -  Height   -      rate      '
   printf,unitout,'#    (c200)     -   (M200)    -  (sigma)   -   (nu)    -   (dM200/dt)   '
   printf,unitout,'#               - Log10[Msun] -            -           - Log10[Msun/yr] '

;; Create a look up table for power spectrum and rms density field
    starttime_table = systime(1)
    print,'Creating sigma look-up table'
    fill_sigma_table
    endtime_table = systime(1)
    print, "sigma table took ", endtime_table-starttime_table, " seconds"

    jmax = long((Mmax-Mmin)/step)
    Mj = Mmin
    
    for j = 0,jmax[0] do begin
        params = COM(Mj,z0)
        cj = params[0]                      ;Concentration c200
        sigma = params[1]                   ; rms density field
        nu = params[2]                      ; peak height
        dMdtj = accretion_rate(Mj,z0,z0)    ; Accretion rate in units of Log10[Msun/yr]
        printf, unitout, cj, Mj, sigma,nu,dMdtj,format='(4X,f0.5,8X,f0.5,6X,f0.5,6X,f0.5,5X,f0.5)'
        Mj += step
    endfor
    close,/all
    endtime = systime(1)
    print, "Determining concentration-mass relation took ", endtime-starttime, " seconds"

endif
;% ----------------------------------------------------




stop
END
