pro insert_cosmology
; ------------------------------------------------------
; Sends a range of cosmological values based on 
; the user input and distribute them
; to the common block: cosmovalues
; ------------------------------------------------------
common cosmovalues,Om,Ol,h,n,sigma8,Aparam

     print,'Your cosmology is not loaded in this program.'
     print,'Insert your cosmological parameters.'
     print,'Insert Om:'
     read,Om
     Ol = 1.-Om
     print,'Insert h:'
     read,h
     print,'Insert sigma8:'
     read,sigma8
     print,'Insert ns:'
     read,n
     Aparam = 0. ;We don't know a best-fit parameter yet.
return
end
; ------------------------------------------------------
function check_cosmo,which_cosmo
   check = 'False'
   all_cosmo = ['WMAP1','WMAP3','WMAP5','WMAP9','Planck']
   for j=0,n_elements(all_cosmo)-1 do begin
       if all_cosmo[j] eq which_cosmo then check='True'
   endfor
return,check
end
; ------------------------------------------------------

pro cosmology, which_cosmo
; ------------------------------------------------------
; Outputs a range of cosmological values based on 
; the header information from the input file and sent
; them to the common block: cosmovalues
; ------------------------------------------------------
common cosmovalues,Om,Ol,h,n,sigma8,Aparam
   check = check_cosmo(which_cosmo)
   if check eq 'True' then begin
      if which_cosmo eq 'WMAP1' then begin
        Om = 0.25
        Ol = 0.75
        h = 0.73
        sigma8 = 0.9
        n = 1.0 
        Aparam = 853.
      endif 
      if which_cosmo eq 'WMAP3' then begin
        Om = 0.238
        Ol = 0.762
        h = 0.73
        sigma8 = 0.74
        n = 0.951
        Aparam = 850.
      endif
      if which_cosmo eq 'WMAP5' then begin
        Om = 0.258
        Ol = 0.742
        h = 0.72
        sigma8 = 0.796
        n = 0.963
        Aparam = 887.
      endif
      if which_cosmo eq 'WMAP9' then begin
        Om = 0.282
        Ol = 0.718
        h = 0.70
        sigma8 = 0.817
        n = 0.964
        Aparam = 950.
      endif
      if which_cosmo eq 'Planck' then begin
        Om = 0.317
        Ol = 0.683
        h = 0.67
        sigma8 = 0.8234  ;!
        n = 0.962
        Aparam = 880.
      endif
   endif else begin
     insert_cosmology
   endelse
   return
END
; ------------------------------------------------------

function intGrowth,z,Om,Ol
; ------------------------------------------------------
;/* Returns the integral from z to z+100 */
;/  (z=zz+dz/2.;z<zz+100.;z+=dz) 
; ------------------------------------------------------
   dz = 0.001  
   intg = 0.
   zint = z
   jfinal = 100./dz
   for j=0,jfinal do begin
    Hz = sqrt(Om*(1.+zint)*(1.+zint)*(1.+zint)+Ol)
    intg += (1.+zint)/Hz/Hz/Hz
    zint += dz 
   endfor
   intg=intg*dz
  return, intg
end

function GrowthFactor,z,Om,Ol
; ------------------------------------------------------
;/*  Calculates the growth factor for a given redshift z
;/*  and cosmology Omega_m : Om, Omega_Lambda = Ol 
; ------------------------------------------------------
  Hz = sqrt(Om*(1.+z)*(1.+z)*(1.+z)+Ol)
  return,Hz*intGrowth(z,Om,Ol)/intGrowth(0.,Om,Ol)
end


function dDdz, z,Om,Ol
; ------------------------------------------------------
;/* Calculates the derivative of the growth factor:dD/dz
;/* for a given redshift z and cosmology (Om,Ol)
; ------------------------------------------------------
   Hz = (Om*(1.+z)^3.+Ol)^(1./2.)
   fz = (1.+z)/Hz^3. 
   result = (GrowthFactor(z,Om,Ol)/Hz^2)*(3./2.)*Om*(1.+z)^2
   result -= fz*GrowthFactor(z,Om,Ol)/intGrowth(z,Om,Ol)
   return,result
end


function TransferFunction,k
common cosmovalues,Om,Ol,h,n,sigma8,Aparam
; ------------------------------------------------------
;/* It calculates the transfer fraction from Eisenstein and Hu,
;/* assuming no neutrinos
; ------------------------------------------------------

;%/ -----------------------------------------------------
;%/* In case we assume the Bardeen et al. 1986 transfer 
;%/  function: */
     Ob = 0.045
;%   Gamma = Om*h*exp(-Ob*(1.+(2.*h)^(0.5)/Om))   
;%   q = k/Gamma
;%   T = (alog(1.+2.34*q)/(2.34*q))*(1.+3.89*q+(16.1*q)^2.+(5.46*q)^3.+(6.71*q)^4.)^(-1./4.)
;%/ -----------------------------------------------------
 
;% -----------------------------------------------------
;% Define variables:
;%   fc,fcb,fb,fnub;  /* Fraction of Omega0 in cold dm, cdm+baryon */
;%   pc,pcb;          /* Functions of the f's */
  hh = h*h
  Omh = Om*hh
  Obh = Ob*hh
  b1 = 0.313*Omh^(-0.419)*(1.+0.607*Omh^(0.674))
  b2 = 0.238*Omh^(0.223)
;%  Theta27 = 1.0093  ;%/* The temperature of the CMB, in units of 2.7 K */
  Theta27 = 2.728/2.7   ;%/* Assuming T_cmb = 2.728 K */

;%  /* Compute the equality scale. */
  TTheta27 = Theta27*Theta27
  z_equality = 24986.8*Omh/(TTheta27*TTheta27)     ;%/* Actually 1+z_eq; /* Redshift of matter-radiation equality */
  k_equality = 0.0746*Omh/TTheta27   ;%/* The comoving wave number of the horizon at equality*/

;%  /* Compute the drag epoch and sound horizon */
  z_drag = 1291.*Omh^(0.251)/(1+0.659*Omh^(0.828))
  z_drag *= (1.+b1*Obh^(b2))    ;%/* Redshift of the drag epoch */
  y_drag = z_equality/(1+z_drag)            ;%/* Ratio of z_equality to z_drag */

  fb  = Ob/Om
  fcb = 1.      ;%/* CDM fraction */
  fc  = fcb-fb  ;%/* Baryon + CDM fraction */
  fnub = fb
;%  /* Set up for the free-streaming & infall growth function */
  pcb = 0.      ;%/* The correction to the exponent after drag epoch */
  pc  = 1.25-sqrt(1.+24.*fc)/4.   ;%/* The correction to the exponent before drag epoch */
;%  /* Compute small-scale suppression */  
  alphan = (fc/fcb)*(5.-2.*(pc+pcb))/(5.-4.*pcb)
  alphan *= (1.+fnub*(-0.533+0.126*fnub*fnub))*(1.+y_drag)^(pcb-pc)
  alphan *= (1.+((pc-pcb)/2.)*(1.+1./((3.-4.*pc)*(7.-4.*pcb)))/(1.+y_drag))

  betac = 1./(1.-0.949*fb)       ;%/* The correction to the log in the small-scale */
  alpha_gamma = sqrt(alphan)     ;%/* sqrt(alpha_nu) */
  R_drag = 31.5*Obh*1000./((1.+z_drag)*TTheta27*TTheta27)
  R_equality = 31.5*Obh*1000./((1.+z_equality)*TTheta27*TTheta27)
  sound_horizon_fit = (2./(3.*k_equality))*sqrt(6./R_equality)  ;%/* The sound horizon at the drag epoch */
  sound_horizon_fit *=  alog((sqrt(1.+R_drag)+sqrt(R_drag+R_equality))/(1.+sqrt(R_equality)))

;%/* CDM power spectrum based on the transfer function 
;%   from Eisenstein and Hu (assuming no neutrinos) */
;%   Input k is the wavenumber in units of 1/Mpc   
;% q;   /* The scaled wavenumber */
;% c,l; /* As defined by Eisenstein and Hu */
;% T;   /* The transfer function */

  gamma_eff = Omh*(alpha_gamma+(1-alpha_gamma)/(1+(k*sound_horizon_fit*0.43)^(4.)))
  q = k*TTheta27/gamma_eff
  t2 = q^(1.11)
  c = 14.4+(325./(1.+60.5*t2));
  l = alog(exp(1.)+1.84*betac*alpha_gamma*q)
  T = 1./(1.+c*q*q/l)
;%  return k*(T*PI)^2./8.
  return,T
end

function window,x
; ------------------------------------------------------
;% Gives the window function:
  return,3.*(sin(x)-x*cos(x))/(x*x*x)
end


function CobeNorm,n,Ol,Om
; ------------------------------------------------------
;% We normalize the Power spectrum to the COBE measurements.
;% We use the fitting formulae from Bunn & White 97'
; ------------------------------------------------------
     if n ge 0.7 then begin
        if Ol gt 0 then begin
          a = -0.785-0.05*alog(Om)
          b = -0.95*(n-1.)-0.169*(n-1.)*(n-1.)
          deltaH = 1.94E-5*Om^(a)*exp(b)
        endif else begin
        ;%if n ge 0.7 and Ol eq 0 then begin
          a = -0.35-0.19*alog(Om)-0.17*(n-1.)
          b = -1.*(n-1)-0.14*(n-1)*(n-1)
          deltaH = 1.95E-5*Om^(a)*exp(b)
        endelse
     endif else begin
        if Ol gt 0 then begin
          a = -0.785-0.05*alog(Om)
          b = (n-1.)+1.97*(n-1.)*(n-1.)
          deltaH = 1.94E-5*Om^(a)*exp(b)
        endif else begin
        ;%if n lt 0.7 and Ol eq 0 then begin
          a = -0.35-0.19*alog(Om)-0.15*(n-1.)
          b = -1.02*(n-1)-1.70*(n-1)*(n-1)
          deltaH = 1.95E-5*Om^(a)*exp(b)
        endelse
     endelse
return,deltaH
end

function Power_nonorm,k 
common cosmovalues,Om,Ol,h,n,sigma8,Aparam
;% -----------------------------------------------------
;% This function calcualtes the power spectrum without the final
;% normalization due to sigma8
;% -----------------------------------------------------
    deltaH = CobeNorm(n,Ol,Om)  ;%/* Cobe Norm */
    c = 9.71560348808e-15       ;% Speed of light Mpc/s
    H100_s = h*3.24077648681e-18  ;% H100_s: 100 km s^-1 Mpc^-1 in s^-1
    cH = c/H100_s
    cte = 2.*!pi*!pi*(cH*cH*cH)*deltaH*deltaH   ;% Gives units of Mpc^3
    ;%//k in units of Mpc^-1 
    Pinitial = cte*(k*cH)^n
    if Ol eq 0.0 then P = Pinitial else begin
    T = TransferFunction(k)
    P = T*T*Pinitial
    endelse
    ;%//power in units of de Mpc^3
return,P
end

function sig8Integral,kin
common cosmovalues,Om,Ol,h,n,sigma8,Aparam
;% -----------------------------------------------------
;% Gives the function inside integral assuming R = 8/h Mpc:
;% -----------------------------------------------------
   k = 10^(kin)
;% k in units Mpc^-1
   u = k*8./h
   dk = k*alog(10.)
   sig8 = Power_nonorm(k)*k*k*window(u)*window(u)*dk  ;%last *dk Because change variables x=log_10(k)
return,sig8
end

function normalize,x
common cosmovalues,Om,Ol,h,n,sigma8,Aparam
;% -----------------------------------------------------
;%/* This routine normalizes the power spectrum using sigma 8 
;%   as provided in input */
;%/* sigma 8 here as calculated by the integral */
;%/ I include a short wavelenght cutoff in limit_max:
;% -----------------------------------------------------
     if Ol gt 0. then limit_max = alog10(1E6) else begin
        if n eq 1.0 then limit_max = alog10(1E2)
        if n eq 0.0 then limit_max = alog10(1E2)
        if n eq -1.0 then limit_max = alog10(1E6)
        if n eq -2.0 then limit_max = alog10(1E6)
     endelse
  sig8Integral = qromb('sig8Integral',alog10(1E-8),limit_max)
  sig8=sqrt(sig8Integral/(2.*!pi*!pi))
  return,sigma8/sig8
end

function Power,k,z,Om_input,Ol_input,h_input,n_input,sigma8_input
common cosmovalues,Om,Ol,h,n,sigma8,Aparam
;% -----------------------------------------------------
;% Function that outputs the power spectrum
;% -----------------------------------------------------
     Om = Om_input
     Ol = Ol_input
     h = h_input
     n = n_input
     sigma8 = sigma8_input
;% ------------------------------
;% Give the power spectrum:
    norm = normalize(sigma8)
    if z eq 0. then D = 1. else D = GrowthFactor(z,Om,Ol)
    P = norm*norm*Power_nonorm(k)*D*D
    ;%//power in units of Mpc^3
return,P
end

function sigIntegral,kin
common cosmovalues,Om,Ol,h,n,sigma8,Aparam
common redshift, z
common radius, R   ;%Radius [Mpc]
;% -----------------------------------------------------
;% Gives the function inside integral assuming radius = R Mpc:
;% -----------------------------------------------------
   k = 10^(kin) 
;% k units in Mpc^-1 :
   u = k*R
   P = Power_nonorm(k)
   dk = k*alog(10.)
   sig8 = P*k*k*window(u)*window(u)*dk  ;%last *dk Because change variables x=log_10(k)
return,sig8
end


function Radio,M
common cosmovalues,Om,Ol,h,n,sigma8,Aparam
common redshift,z
;% -----------------------------------------------------
;% Calculates the radius of a given mass M assuming an overdensity
;% definition of Delta = rho_mean
;% -----------------------------------------------------
   rho0 = 2.775E11*h^2                          ;% Critical density of the universe today! in [Msun/Mpc^3]
   rho = alog10(Om*rho0)+alog10((1.+z)*(1.+z)*(1.+z))      ;%Mean background density Units! Msun/Mpc^3
   R = (1./3.)*alog10(3./(4.*!pi))-(1./3.)*rho+(1./3.)*M   ;%Units! Mpc and Mass units of Msun
return,10^R
end

function sigma, M,z_input,Om_input,Ol_input,h_input,n_input,sigma8_input,norm_input
common cosmovalues,Om,Ol,h,n,sigma8,Aparam
common redshift,z
common radius, R   ;%Radius [Mpc]
;% -----------------------------------------------------
;% Calculates the rms of the density perturbation field
;% -----------------------------------------------------
     z = z_input
     Om = Om_input
     Ol = Ol_input
     h = h_input
     n = n_input
     sigma8 = sigma8_input
     norm = norm_input
;% -----------------------
     R = Radio(M)
     ;%/ I include a short wavelenght cutoff in limit_max:
     if Ol gt 0. then limit_max = alog10(1E6) else begin
        if n eq 1.0 then limit_max = alog10(1E2)
        if n eq 0.0 then limit_max = alog10(1E2)
        if n eq -1.0 then limit_max = alog10(1E6)
        if n eq -2.0 then limit_max = alog10(1E6)
     endelse
     sigIntegral = qromb('sigIntegral',alog10(1E-6),limit_max)
     sigma = sqrt(norm*norm*sigIntegral/(2.*!pi*!pi))
return,sigma
end


function sigmar,R_input,z_input,Om_input,Ol_input,h_input,n_input,sigma8_input
common cosmovalues,Om,Ol,h,n,sigma8,Aparam
common redshift,z
common radius, R   ;%Radius [Mpc]
;% -----------------------------------------------------
;% Caculates the rms of the density perturbation field
;% given a radius R
;% -----------------------------------------------------
     z = z_input
     Om = Om_input
     Ol = Ol_input
     h = h_input
     n = n_input
     sigma8 = sigma8_input
     R = R_input
;% -----------------------
     norm = normalize(sigma8)
     ;%/ I include a short wavelenght cutoff in limit_max:
     if Ol gt 0. then limit_max = alog10(1E6) else begin
        if n eq 1.0 then limit_max = alog10(1E2)
        if n eq 0.0 then limit_max = alog10(1E2)
        if n eq -1.0 then limit_max = alog10(1E6)
        if n eq -2.0 then limit_max = alog10(1E6)
     endelse
     sigIntegral = qromb('sigIntegral',alog10(1E-6),limit_max)
     sigma = sqrt(norm*norm*sigIntegral/(2.*!pi*!pi))
return,sigma
end

function push, array, append
;% -----------------------------------------------------
;% Let's push this array!
;% -----------------------------------------------------
  if n_elements(array) eq 0 then array=[append] else array=[array,append]
return,array
end


pro fill_sigma_table
common cosmovalues,Om,Ol,h,n,sigma8,Aparam
common table,sig_table,M_table
;% -----------------------------------------------------
;% This is a look-up table that calculates power spectrum
;% and sigma and saves them in memory
;% -----------------------------------------------------
   MaxM = 16.
   MinM = -4.
   step = 0.05
   num = (MaxM-MinM)/step
;% Calculate power spectrum correct normalization 
   norm = normalize(sigma8)
   for j = 0,num do begin
       Mj = j*step+MinM
       M_table = push(M_table,Mj)
       sigj = sigma(Mj,0.,Om,Ol,h,n,sigma8,norm)
       sig_table = push(sig_table,sigj)
   endfor
return
end

;%;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
function fitline, B, X=x, Y=y   ;%, ERR=err
model = B[0]*x+B[1]
return, (y-model)    ;%/err
END
;%;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
function recta,a,b,x
model = a*x+b
return, model
END
;%;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

function sigma_table,M
common table,sig_table,M_table
;% -----------------------------------------------------
;% This is a look-up table that calculates power spectrum
;% and sigma and saves them in memory
;% -----------------------------------------------------
   select = where(M_table ge M-0.1 and M_table le M+0.1)
   A = [1,1]
   fa = {X:M_table[select],Y:sig_table[select]}
   fit = mpfit('fitline',A,functargs=fa,/QUIET)
   sig_mean = recta(fit[0],fit[1],M)
return,sig_mean
end


pro cosmo_routines
END
