function accretion_rate,M0,z,zi
common cosmovalues,Om,Ol,h,n,sigma8,Aparam
;/ Routine that computes accretion rate given the halo mass
;/ at redshift zi, M0, and the output redshift, z, for the 
;/ accretion rate. Note that zi<z always.
;% --------------------------------------------
;/ It calculates the parameters alpha (a) and beta (b)
;/ following eqs. (9) and (10) from Correa et al. (2014b).
   zf = -0.0064*M0^2.+0.02373*M0+1.8837
   q = (10^0.6167)*zf^(-0.9476)
   S0 = sigma_table(M0)^2.
   Sq = sigma_table(M0-alog10(q))^2.
   f = (Sq-S0)^(-0.5)
;% --------------------------------------------
   Dz0 = GrowthFactor(zi,Om,Ol)
   a = (sqrt(2./!pi)*1.686*dDdz(zi,Om,Ol)/Dz0^2.+1.)*f
   b = -1.*f
;% --------------------------------------------
;/ Accretion rate at z:
   Mz = M0+alog10((1.+z-zi)^a*exp(b*(z-zi)))
   dMdt = alog10(71.59*(h/0.7)*(-1.*a/(1.+z-zi)-b)*(10^(Mz-12.))*(1.+z)*sqrt(Om*(1.+z)^3.+Ol))
return,dMdt
END
;% ----------------------------------------------------------
function MAH,M0,z,zi
common cosmovalues,Om,Ol,h,n,sigma8,Aparam
;/ Routines that computes halo mass history, M(z) or MAH,
;/ given the halo mass at redshift zi and output redshift z.
;/ Note that zi is the redshift at which the halo MAH begins
;/ being tracked so zi<z always.
;% --------------------------------------------
;/ It computes alpha and beta parameters from MAH following
;/ eqs (8), (9) and (10) from Correa et al. (2014b).
   zf = -0.0064*M0^2.+0.02373*M0+1.8837
   q = (10^0.6167)*zf^(-0.9476)
;% --------------------------------------------
   S0 = sigma_table(M0)^2.
   Sq = sigma_table(M0-alog10(q))^2.
   f = (Sq-S0)^(-0.5)
;% --------------------------------------------
   Dz0 = GrowthFactor(zi,Om,Ol)
   a = (sqrt(2./!pi)*1.686*dDdz(zi,Om,Ol)/Dz0^2.+1.)*f
   b = -1.*f
;% --------------------------------------------
   Mz = M0+alog10((1.+z-zi)^(a)*exp(b*(z-zi))) 
return,Mz
END
;%-----------------------------------------------------
function cduffy,M0,z0
;/ Small routine that return Duffy et al. (2008) c-M relation.
cduffy = 6.71*(10^M0/(2E12/0.72))^(-0.091)*(1.+z0)^(-0.44)
return,cduffy
end
;%-----------------------------------------------------
function COM,M0,z
common cosmovalues,Om,Ol,h,n,sigma8,Aparam
;/ Knowing halo mass and redshift, it performs a Levenberg-Marquardt 
;/ least-squares method, using mpfitfun routine, to calculate c. 
;/ The subroutine which_c? uses eqs. (17) and (18) from the paper. 
    Mini = M0
    zini = z
;% --------------------------------------------
   zf = -0.0064*M0^2.+0.02373*M0+1.8837
   q = (10^0.6167)*zf^(-0.9476)
   S0 = sigma_table(M0)^2.
   Sq = sigma_table(M0-alog10(q))^2.
   f = (Sq-S0)^(-0.5)
;% --------------------------------------------
   Dz0 = GrowthFactor(zini,Om,Ol)
;/ alpha and beta parameters:
   a_tilde = (sqrt(2./!pi)*1.686*dDdz(zini,Om,Ol)/Dz0^2.+1)*f
   b_tilde = -1.*f
;% --------------------------------------------
;/  mpfitfun routine was written by Craig B. Markwardt.
    result = mpfitfun('which_c',[Mini,zini,a_tilde,b_tilde],[0,0],1,[5d],/quiet)
    sig = sigma_table(M0)
    nu = 1.686/(sig*GrowthFactor(z,Om,Ol))
return,[result,sig,nu]
end
;%-----------------------------------------------------
function delta,x
common cosmovalues,Om,Ol,h,n,sigma8,Aparam
    rho0 = alog10(2.775)+11. ;[h^2 Msun/Mpc^3]
    rhom = alog10(Om)+rho0
    M8 = alog10((4*!pi/3.)*8.^3.)+rhom   ;[h^-1 Msun]
    M8 -= alog10(h)  ;[Msun]
    k = (1./3.)*(x-M8)
    k = 10^k
    delta = (0.796/sigma8)*k^((0.963-n)/2.)
return,1.
end
;%-----------------------------------------------------
function which_c,param,x
common cosmovalues,Om,Ol,h,n,sigma8,Aparam
;/ Routine called by mpfitfun to solve 2 eqs. (17 and 18 from 
;/ Correa et al. 2014b) with 1 unkown: concentration.
;%-----------------------------------------------------
   M0 = param[0]
   z0 = param[1]
   a_tilde = param[2]
   b_tilde = param[3]
;% --------------------------------------------
   ;% function 1:
   Y1 = alog(2)-0.5
   Yx = alog(1.+x)-x/(1.+x)
   f1 = alog(Y1/Yx)
   ;% --------------------------------------------
   ;% function 2:
   rho_2 = 200.*x^3.*Y1/Yx
;% I insert a delta_sigma propto formation time (see Correa et
;  al. 2014a) to convert best-fit parameter of rho_crit-rho_2
;  relation between cosmologies:
   if Aparam eq 0. then begin
      delta_sigma = delta(M0)
      bestfitparam = 887.*delta_sigma 
   endif else begin
   bestfitparam = Aparam
   endelse
   zf = ((rho_2*(Om*(1.+z0)^3.+Ol)/bestfitparam)/Om-Ol/Om)^(1./3.)-1.
   f2 = (a_tilde*alog(1.+zf-z0)+b_tilde*(zf-z0))
return,f1-f2
end
;%-----------------------------------------------------
function Integrate_rho, x  
common halo_conc,cin
   c = cin
   rho = 1./(x*c*(1.+x*c)^2.)
   int = x^2.*rho^2
   return,int
end 
;% ---------------------------------------------------
function r200,M,z
;/ Routine that calculates r200:
rho0 = alog10(2.775)+11. ;[h^2 Msun/Mpc^3]
rho = rho0+alog10(0.258*(1.+z)^3.+0.742)
r200 = M+alog10(3./(4.*!pi*200.))-rho
r200 *= (1./3.)
return,r200   ;[h^-1 Mpc]
end
;% ---------------------------------------------------
function Int_rho,M,c,z  
common halo_conc,cin
      cin = c
;% We calculate Int(rho^2 dV) ------------------------
      Int_rho_dV = QROMB('integrate_rho',0.001,1.)
      Rvir = r200(M,z)   ; [h^-1 Mpc]
      Yc = alog(1.+c)-c/(1.+c)
      delta_c = alog10((200./3.)*c^3/Yc)
      rho0 = alog10(2.775)+11. ;[h^2 Msun/Mpc^3]
      rho_s = rho0+alog10(0.258*(1.+z)^3.+0.742)+delta_c   ;[h^2 Msun/Mpc^3]
      Int = alog10(4.*!pi)+Rvir+2.*rho_s+alog10(Int_rho_dV)
   return,Int
end 

;%--------------------------------------------------
pro subroutines

END
